/**
* @file        main.c
* @brief       This file provides all the basic main firmware functions.
* @author      ChipSea MCU Group
* @version     0.0.1
* @date        2023.03.22
* @copyright   CHIPSEA TECHNOLOGIES (SHENZHEN) CORP.
* @note        
****************************************************************************************************
* @par Modify record:
* <tr><th>Date          <th>Version    <th>Author     <th>Description
* <tr><td>2023.03.22    <td>0.0.1     <td>ChipSea    <td>
* -# 
* </table>
*
****************************************************************************************************
*/


/***************************************************************************************************
 * @includes
***************************************************************************************************/
#include "include.h"
#include "canfestival.h"


#include "Slave_f103.h"
#include "node_config.h" 
#include "slave_callback.h"

/***************************************************************************************************
 * @definitions
***************************************************************************************************/
static int slave_callback(void);


/** @addtogroup main
  * @{
  */

/** @defgroup main
  * @brief    main driver modules
  * @{
  */

///@cond

/***************************************************************************************************
 * @variables
***************************************************************************************************/


/***************************************************************************************************
 * @function
***************************************************************************************************/
int main(void)  
{
    nvic_priority_group_config(NVIC_PriorityGroup_4); 
    
    bsp_init(); 
    
    /* CAN init PA11 RX / PA12 TX  500Kbps*/
    can_mode_config(CAN_TIME_CHANGE_UNIT_1,CAN_TS1_UNIT_9,CAN_TS2_UNIT_8,4,CAN_MODE_NORMAL);

    //timerscfg.hMS_TO_TIMEVAL/US_TO_TIMEVAL,ӻΪ6553.5ms
    initTimer(); 
    
    slave_callback();
    
    printf("CanOpen Slave Node Test...\r\n\n");
    
    unsigned char nodeID = SLAVE_NODE_ID; 
    
    setNodeId(&Slave_Data, nodeID);
    
    //InitialisationPre-operationalתӽڵԶ
    setState(&Slave_Data, Initialisation); 
    
    //ҪNMT ӽڵ㣬ӻɷSDO,PDO
//    setState(&Slave_Data, Operational); 

    while(1)
    {
        if(can_rec_flag) 
        {
            can_rec_flag = FALSE;
            
            __TIM_INTR_DISABLE(TIM4, TIM_INTR_UPDATE);  
            
            canDispatch(&Slave_Data, &canopen_msg); 
            
            __TIM_INTR_ENABLE(TIM4, TIM_INTR_UPDATE);                        
        }
        
        led1_toggle(); 
        led2_toggle();
        delay(0xFFFFFF);
        
        {
            Temperture[0] +=1; //ֵֵ,ͨPDO 
            
            if(getState(&Slave_Data) == 0x5)
            {
                Message temperture_value;
                temperture_value.cob_id = 0x281;
                temperture_value.rtr = 0;
                temperture_value.len = 4;
                memcpy(temperture_value.data, Temperture, temperture_value.len);
                canSend((&Slave_Data)->canHandle,&temperture_value);
            }
           else
           {
                UNS32 size = sizeof(UNS32);
                UNS16 index = 0x2000;
                UNS32 value = Temperture[0];
                UNS8 subindex=1;
		        writeLocalDict( &Slave_Data, /*CO_Data* d*/
		    								index,       /*UNS16 index*/
		    								subindex,         /*UNS8 subind*/ 
		    								&value,  /*void * pSourceData,*/ 
		    								&size,        /* UNS8 * pExpectedSize*/
		    								RW);          /* UNS8 checkAccess */
           }    
    
            
        }
        sdo_test[0] = 0xAA; //SDO,Mapped at index 0x2002, subindex 0x01 - 0x01 -> 4f 02 20 01 00 00 00 00  
        
        printf("TempertureֵΪ%x...\r\n",Temperture[0]); 
    }

}


static int slave_callback(void)
{
		
    Slave_Data.heartbeatError = Slave_Data_heartbeatError; //ֵԭʼֵΪ _heartbeatErrorǸպ,
                                                            //󣬴ӡϢ 
    Slave_Data.initialisation = Slave_Data_initialisation; //user app init callback
    Slave_Data.preOperational = Slave_Data_preOperational;
    Slave_Data.operational = Slave_Data_operational;
    Slave_Data.stopped   = Slave_Data_stopped;
    Slave_Data.post_sync = Slave_Data_post_sync;     //յͬappҪ
    Slave_Data.post_TPDO = Slave_Data_post_TPDO;     //TPD0appҪ ,ͬtpdO
    Slave_Data.storeODSubIndex = Slave_Data_storeODSubIndex;
    Slave_Data.post_emcy = Slave_Data_post_emcy;     //ӻյϢappҪ
	
	return 0;
}

#ifdef  USE_FULL_ASSERT

/**@brief       Report the assert error.
 *
 * @param[in]   file: pointer to the source file name.
 *
 * @param[in]   line: error line source number.
 *
 * @return      None.
 */
void assert_failed(uint8_t* file, uint32_t line)
{
    while(1);
}

#endif

///@endcond

/**
  * @}
  */

/**
  * @}
  */
