/***************************************************************************//**
 * @file        cs32f10x_it.c
 * @version     V2.0.1
 * @author      Software Development
 * @brief       This file provides all the interrupt handle functions.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
 ****************************************************************************/
 
#include "include.h"
#include "cs32f10x_it.h"
#include "bsp_can.h"

/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**@brief       This function handles NMI exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void NMI_Handler(void)
{
    /* User code */
}

/**@brief       This function handles Hard Fault exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void HardFault_Handler(void)
{
  while (1)
  {
      /* User code */
  }
}

/**@brief       This function handles Memory Manage exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void MemManage_Handler(void)
{
  while (1)
  {
      /* User code */
  }
}

/**@brief       This function handles Bus Fault exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void BusFault_Handler(void)
{
  while (1)
  {
      /* User code */
  }
}

/**@brief       This function handles Usage Fault exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void UsageFault_Handler(void)
{
  while (1)
  {
      /* User code */
  }
}

/**@brief       This function handles SVCall exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
//void SVC_Handler(void)
//{
//    /* User code */
//}

/**@brief       This function handles Debug Monitor exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
void DebugMon_Handler(void)
{
    /* User code */
}

/**@brief       This function handles PendSVC exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
//void PendSV_Handler(void)
//{
//    /* User code */
//}

/**@brief       This function handles SysTick exception.
 *
 * @param[in]   None.
 *
 * @return      None.
 */
//void SysTick_Handler(void)
//{
//    /* User code */
//}

/******************************************************************************/
/*                 CS32F10x Peripherals Interrupt Handlers                   */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
#if configGENERATE_RUN_TIME_STATS
void TIM3_IRQHandler(void)
{
    if(__TIM_INTR_STATUS_GET(TIM3, UPDATE))
    {
        FreeRTOSRunTimeTicks++;
    }
    __TIM_FLAG_CLEAR(TIM3, TIM_FLAG_UPDATE);
}

#else

void TIM3_IRQHandler(void)
{
    if(__TIM_INTR_STATUS_GET(TIM3, UPDATE))
    {
        printf("TIM3 INTR.......\r\n");
    }
    __TIM_FLAG_CLEAR(TIM3, TIM_FLAG_UPDATE);
}
#endif


/**@brief       Can festival timer -- TIMER 4
 * 
 * @param[in]   arr: period.
 * 
 * @param[in]   psc: prescale.
 * 
 * @return      None.
 */

extern TIMEVAL last_counter_val;
extern TIMEVAL elapsed_time;

void TIM4_IRQHandler(void)
{
#if OS_ENABLE
    uint32_t status_value = 0;
    status_value = taskENTER_CRITICAL_FROM_ISR();
#endif
    if(__TIM_INTR_STATUS_GET(TIM4, UPDATE))
    {
        last_counter_val = 0;
        elapsed_time = 0;
        __TIM_FLAG_CLEAR(TIM4, TIM_FLAG_UPDATE);
        
        /* canfestival dispatch */
        TimeDispatch();
    }
#if OS_ENABLE
    taskEXIT_CRITICAL_FROM_ISR(status_value);
#endif
}

/**@brief       This function handles CAN1 interrupt request
 * 
 * @param[in]   None.
 * 
 * @return      None.
 */
void USB_LP_CAN1_RX0_IRQHandler(void)
{
    can_rx_msg_t rx_msg;
#if OS_ENABLE
    BaseType_t err;
    BaseType_t xHigherPriorityTaskWoken;
#endif
    
    can_message_receive(CAN1, CAN_RECEIVE_FIFO_0,&rx_msg);
    
    /* Assembly canopen data packet */
    canopen_msg.cob_id = rx_msg.std_id;
    canopen_msg.rtr    = rx_msg.remote_txreq;
    canopen_msg.len    = rx_msg.data_length;
    memcpy(canopen_msg.data, rx_msg.data, rx_msg.data_length);

    printf("յCANopen...\r\n");
    can_rec_flag = TRUE;
    
#if OS_ENABLE
    err = xQueueSendFromISR(Message_Queue,&rx_message, &xHigherPriorityTaskWoken);
    if(err == errQUEUE_FULL)
    {
        printf("Key_Queue send fail because of full\r\n");
    }
    
    // Now the buffer is empty we can switch context if necessary.
    if( xHigherPriorityTaskWoken )
    {
        // Actual macro used here is port specific.
        portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
    }
#endif
}



/**
  * @}
  */ 


/******************* (C) COPYRIGHT 2021 ChipSea *****END OF FILE****/
