#include "bsp_can.h"
#include "include.h"

#include <stdio.h>
#include "cs32f10x_can.h"
#include "cs32f10x_usart.h"
#include "cs32f10x_gpio.h"
#include "cs32f10x_rcu.h"
#include "cs32f10x_misc.h"
#include "cs32f10x_tim.h"

#include "canfestival.h"

#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "croutine.h"

/* CanOpen dictionary */
CO_Data masterObjdict_Data;

TIMEVAL last_counter_val = 0;
TIMEVAL elapsed_time = 0;

volatile unsigned char  can_rec_flag=0;
Message canopen_msg;

extern QueueHandle_t Message_Queue;        //ϢϢо


/**@brief       CAN mode init and config.
 *
 * @param[in]   tc: time_change, Specifies the maximum number of time unit the CAN module
 *              could be changed lengther or shorter.
 *
 * @param[in]   ts1: Time segment 1.
 *
 * @param[in]   ts2: Time segment 2.
 *
 * @param[in]   brp: CAN baud rate prescaler division. 
 *              CAN_BAUD = pclk/((ts1+1 + ts2+1 +1)*brp)   36M/((8+9+1)*4)=500Kbps
 *
 * @param[in]   mode: CAN work mode.CAN_MODE_NORMAL or CAN_MODE_LOOPBACK in this case.
 *
 * @return      None.
 */
uint8_t can_mode_config(uint32_t tc, uint32_t ts1, uint32_t ts2, uint32_t brp,uint32_t mode)
{
    can_config_t can_config = {0};
    can_filter_config_t can_filter = {0};
    nvic_init_t nvic_struct ;
    
    __RCU_APB1_CLK_ENABLE(RCU_APB1_PERI_CAN1);
    __RCU_APB2_CLK_ENABLE(RCU_APB2_PERI_GPIOA);
    
    /* PA11 RX */
    gpio_mode_config(GPIOA, GPIO_PIN_11, GPIO_MODE_IN_PU);
    /* PA12 TX */
    gpio_mode_config(GPIOA, GPIO_PIN_12, GPIO_MODE_OUT_AFPP(GPIO_SPEED_HIGH));
    
    /* CAN config */
    can_config.time_trigger = DISABLE;
    can_config.auto_busoff = DISABLE;
    can_config.auto_active = DISABLE;
    can_config.no_retransmit = ENABLE;
    can_config.rx_overwrite = ENABLE;
    can_config.tx_priority = DISABLE;
    can_config.can_mode = mode;
    can_config.time_change = tc;
    can_config.time_seg1 = ts1;
    can_config.time_seg2 = ts2;
    can_config.baudrate_pdiv = brp;
    
    can_init(CAN1, &can_config);
    
    /* Filter config */
    can_filter.filter_fifo_connect = CAN_FILTER_CONNECT_FIFO0;  
    can_filter.filter_group = 0;    //filter 0
    can_filter.filter_mode = CAN_FILTER_MODE_IDENTIFY_MASK;
    can_filter.filter_identifier = 0x00000000;
    can_filter.filter_mask = 0x00000000;
    can_filter.filter_active = ENABLE;
    
    can_filter_init(&can_filter);
    
    /* NVIC config */
    __CAN_INTR_ENABLE(CAN1, CAN_INTR_FF0_MSG_PENDING);
    nvic_struct.nvic_irqchannel = IRQn_USB_LP_CAN1_RX0;
    nvic_struct.nvic_irq_enable = ENABLE;
    nvic_struct.nvic_irq_pre_priority = 0;  //6?  ȼ 0
    nvic_struct.nvic_irq_sub_priority = 0;
    nvic_init(&nvic_struct);
    
    return 0;
}

// The driver send a CAN message passed from the CANopen stack
unsigned char canSend(CAN_PORT notused, Message *m)
{
    uint8_t mailbox;
    uint16_t i = 0;
    can_tx_msg_t tx_msg;
    
    tx_msg.std_id = m->cob_id;
    tx_msg.ext_id = m->cob_id;
    tx_msg.id_select = CAN_ID_STANDARD;
    tx_msg.remote_txreq = (m->rtr == CAN_RMOTE_TX_REQ_DATA) ? 0:2;
    tx_msg.data_length = m->len;
    
    memcpy(tx_msg.data, m->data, m->len);
    
    mailbox = can_message_send(CAN1, &tx_msg);
//    printf("Test 6...\r\n"); 
//    i = 0;  //i used for timeout
//    while(can_tx_status_get(CAN1,mailbox) == 0x01)   //Ῠ ֵ̫죬CANͣ
//    {
//        i++;
//        if(i == 0XFFF)
//        {
//           printf("Test 4...\r\n");return 1;
//        }
//    }
//    printf("Test 5...\r\n"); 
    return 0;
}


//uint8_t can_send_msg(uint8_t *msg, uint8_t len)
//{
//    uint8_t mailbox;
//    uint16_t i = 0;
//    can_tx_msg_t tx_msg;
//    
//    tx_msg.std_id = 0x33;
//    tx_msg.ext_id = 0x33;
//    tx_msg.id_select = CAN_ID_EXTEND;
//    tx_msg.remote_txreq = CAN_RMOTE_TX_REQ_DATA;
//    tx_msg.data_length = len;
//    
//    for(i = 0; i< len; i++)
//    {
//        tx_msg.data[i] = msg[i];
//    }
//    
//    mailbox = can_message_send(CAN1, &tx_msg);
//    
//    i = 0;  //i used for timeout
//    while(can_tx_status_get(CAN1,mailbox) == 0x01)
//    {
//        i++;
//        if(i == 0XFFF)
//        {
//            return 1;
//        }
//    }
//    
//    return 0;
//}

uint8_t can_receive_msg(uint8_t *rec)
{
    uint32_t i = 0;
    can_rx_msg_t rx_msg;
    
    if(can_fifo_msg_num_get(CAN1, CAN_RECEIVE_FIFO_0) == 0)
    {
        return 0;
    }
    
    can_message_receive(CAN1, CAN_RECEIVE_FIFO_0, &rx_msg);
    
    for(i = 0; i< 8; i++)
    {
        rec[i] = rx_msg.data[i];
    }
    return rx_msg.data_length;
}

/**@brief       Can festival timer -- TIMER 4
 * 
 * @param[in]   arr: period.
 * 
 * @param[in]   psc: prescale.
 * 
 * @return      None.
 */
void tim4_config(uint16_t arr, uint16_t psc)
{
    tim_base_t ptr_time = {0};
    nvic_init_t ptr_nvic;
    
    __RCU_APB1_CLK_ENABLE(RCU_APB1_PERI_TIM4);
    
    /* Time base configuration */
    ptr_time.clk_div = TIM_CLK_DIV1;
    ptr_time.cnt_mode = TIM_CNT_MODE_UP;
    ptr_time.period = arr;
    ptr_time.pre_div = psc;
    tim_base_init(TIM4, &ptr_time);
    
    __TIM_INTR_ENABLE(TIM4, TIM_INTR_UPDATE);
    
    /* Configure and enable TIM4 interrupt. */
    ptr_nvic.nvic_irqchannel = IRQn_TIM4;
    ptr_nvic.nvic_irq_enable = ENABLE;
    ptr_nvic.nvic_irq_pre_priority = 0;
    ptr_nvic.nvic_irq_sub_priority = 0;
    nvic_init(&ptr_nvic);
    
    __TIM_COUNTER_SET(TIM4, 0);
//    __TIM_DISABLE(TIM4);
    __TIM_ENABLE(TIM4);
}

void start_callback(CO_Data* d, UNS32 id)
{
}

void initTimer(void)
{
    /* TIM4 Init 1ms */
    tim4_config(10, 7200-1); //72000000hz/7200=1 000 0hz  max->6553.5ms
    
    // this is needed for correct canfestival virtual timer management start
    SetAlarm(NULL, 0, start_callback, 0, 0);
}


/**@brief       Can festival function.
 *              Set the timer for the next alarm.
 * 
 * @param[in]   None.
 * 
 * @return      None.
 */
void setTimer(TIMEVAL value)
{
    uint32_t timer = __TIM_COUNTER_GET(TIM4);        // Copy the value of the running timer
    elapsed_time += timer - last_counter_val;
    last_counter_val = 65535-value;
    __TIM_COUNTER_SET(TIM4, 65535-value);
    __TIM_ENABLE(TIM4); 
} 


/**@brief       Can festival function.
 *              Return the elapsed time to tell the Stack how much time is spent since last call.
 * 
 * @param[in]   None.
 * 
 * @return      None.
 */
TIMEVAL getElapsedTime(void)
{
    uint32_t timer = __TIM_COUNTER_GET(TIM4);        // Copy the value of the running timer
    if(timer < last_counter_val)
        timer += 65535;
    TIMEVAL elapsed = timer - last_counter_val + elapsed_time;
    //printf("elapsed %lu - %lu %lu %lu\r\n", elapsed, timer, last_counter_val, elapsed_time);
    return elapsed;
}

UNS8 canChangeBaudRate(CAN_PORT port, char* baud)
{
    return 0;
}





