/***************************************************************************//**
 * @file        bsp_can.h
 * @version     
 * @author      Software Development
 * @brief       This file provides can handle functions.
 *              
 * @copyright   Copyright (C) Software Development. All rights reserved.
 ****************************************************************************/
 
#ifndef __BSP_CAN_H__
#define __BSP_CAN_H__

#ifdef __cplusplus
 extern "C" {
#endif 


#include "cs32f10x.h"
#include "canfestival.h" 



/**@brief       CAN mode init and config.
 *
 * @param[in]   tc: time_change, Specifies the maximum number of time unit the CAN module
 *              could be changed lengther or shorter.
 *
 * @param[in]   ts1: Time segment 1.
 *
 * @param[in]   ts2: Time segment 2.
 *
 * @param[in]   brp: CAN baud rate prescaler division. 
 *              CAN_BAUD = pclk/((ts1+1 + ts2+1 +1)*brp)   36M/((8+9+1)*4)=500Kbps
 *
 * @param[in]   mode: CAN work mode.CAN_MODE_NORMAL or CAN_MODE_LOOPBACK in this case.
 *
 * @return      None.
 */
uint8_t can_mode_config(uint32_t tc, uint32_t ts1, uint32_t ts2, uint32_t brp,uint32_t mode);

uint8_t can_send_msg(uint8_t *msg, uint8_t len);

uint8_t can_receive_msg(uint8_t *rec);

extern void initTimer(void);
extern UNS8 canSend(CAN_PORT notused, Message *m);
extern UNS8 canChangeBaudRate(CAN_PORT port, char* baud);
extern void setTimer(TIMEVAL value);
extern TIMEVAL getElapsedTime(void);

extern volatile unsigned char  can_rec_flag;
extern Message canopen_msg;


#endif



